set linesize 32000
set pagesize 1000
set long 2000000000
set longchunksize 1000
set head off;
set verify off;
set termout off;
 
column u new_value us noprint;
column n new_value ns noprint;
column c new_value cs noprint;
 
select name n from v$database;
select user u from dual;
SELECT SYS_CONTEXT('USERENV', 'CON_NAME') c FROM DUAL;

set sqlprompt &ns:&cs:&us>

set head on
set echo on
set termout on
set trimspool on

spool &ns..&cs..adaptiveplanjoinmethod1.log

-- http://docs.oracle.com/cd/E16655_01/server.121/e15858/tgsql_optcncpt.htm#BABGIEJA
-- example of adaptive plan changing the join method
-- from nested loops to hash join

-- table T1 has conditions on it that produce an incorrect cardinality estimate
-- estimate should be lower than reality
-- T1 joins to T2 to retrieve the needed data

drop table t1;
drop table t2;

create table t1
(
conditionc1 number,
conditionc2 number,
joinc number
);

create table t2
(
joinc number,
datac number
);

-- 16 rows in t1 but 
-- one combination has more than o
-- the others

insert into t1 values (1,9,1);
insert into t1 values (2,8,2);
insert into t1 values (3,7,3);
insert into t1 values (4,6,4);
insert into t1 values (5,5,5);
insert into t1 values (6,4,6);
insert into t1 values (7,3,7);
insert into t1 values (8,2,8);

insert into t1 values (9,1,9);
insert into t1 values (9,1,10);
insert into t1 values (9,1,11);
insert into t1 values (9,1,12);
insert into t1 values (9,1,13);
insert into t1 values (9,1,14);
insert into t1 values (9,1,15);
insert into t1 values (9,1,16);

commit;

-- load table t2 with one row per join value

insert into t2 values (1,1);
insert into t2 values (2,1);
insert into t2 values (3,1);
insert into t2 values (4,1);
insert into t2 values (5,1);
insert into t2 values (6,1);
insert into t2 values (7,1);
insert into t2 values (8,1);
insert into t2 values (9,1);
insert into t2 values (10,1);
insert into t2 values (11,1);
insert into t2 values (12,1);
insert into t2 values (13,1);
insert into t2 values (14,1);
insert into t2 values (15,1);
insert into t2 values (16,1);

commit;

-- create index for join

create index t2i on t2(joinc);

-- gather stats

execute dbms_stats.gather_table_stats(NULL,'T1');
execute dbms_stats.gather_table_stats(NULL,'T2');

-- run query

select /*+ gather_plan_statistics */
sum(t2.datac)
from t1, t2
where
t1.conditionc1 > 8 and
t1.conditionc2 = 1 and
t1.joinc=t2.joinc;

select * from table(dbms_xplan.display_cursor(null,null,'ALLSTATS'));

spool off
